﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data.MoveVectorData;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.MoveVectorEncoding.Manipulators
{
    [Item("SingleMoveShiftManipulator", " Shifts one move to a different position.")]
    [StorableClass]
    public class SingleMoveShiftManipulator : MoveVectorManipulator
    {

        [StorableConstructor]
        protected SingleMoveShiftManipulator(bool deserializing) : base(deserializing) { }
        protected SingleMoveShiftManipulator(SingleMoveShiftManipulator original, Cloner cloner) : base(original, cloner) { }
        
        public SingleMoveShiftManipulator() : base() { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new SingleMoveShiftManipulator(this, cloner);
        }

        public static void Apply(IRandom random, MoveVector moves)
        {
            Manipulate(random, moves, random.Next(moves.Length));
        }

        public static void Manipulate(IRandom random, MoveVector moves, int index)
        {
            moves[index].Enabled = true;
            int shift = 0;
            do
            {
                shift = random.Next(-index, moves.Length - index + 1);
            } while (shift == 0);

            var save = moves[index];
            if (shift > 0)
            {
                for(int i = index; i < index + shift; i++)
                {
                    if (i + 1 == index + shift) moves[index] = save;
                    else moves[i] = moves[i + 1];
                } 
            } else {
                for (int i = index; i >= index + shift; i--)
                {
                    if (i == index + shift) moves[index] = save;
                    else moves[i] = moves[i - 1];
                }
            }
        }

        protected override void Manipulate(IRandom random, MoveVector moves)
        {
            Apply(random, moves);
        }
    }
}
